From: "Stephen M. Cameron" <scameron@beardog.cce.hp.com>
To: axboe@kernel.dk
Cc: mikem@beardog.cce.hp.com, akpm@linux-foundation.org,
thenzl@redhat.com, linux-kernel@vger.kernel.org,
smcameron@yahoo.com
Subject: [PATCH 14/16] cciss: do not attempt PCI power management reset method if we know it won't work.
Date: Tue, 03 May 2011 14:54:02 -0500 [thread overview]
Message-ID: <20110503195402.5154.40908.stgit@beardog.cce.hp.com> (raw)
In-Reply-To: <20110503194919.5154.78352.stgit@beardog.cce.hp.com>
From: Stephen M. Cameron <scameron@beardog.cce.hp.com>
Just go straight to the soft-reset method instead.
Signed-off-by: Stephen M. Cameron <scameron@beardog.cce.hp.com>
---
drivers/block/cciss.c | 36 +++++++++++++++++++++++++++++++-----
1 files changed, 31 insertions(+), 5 deletions(-)
diff --git a/drivers/block/cciss.c b/drivers/block/cciss.c
index 7990b98..865484e 100644
--- a/drivers/block/cciss.c
+++ b/drivers/block/cciss.c
@@ -558,7 +558,7 @@ static void __devinit cciss_procinit(ctlr_info_t *h)
#define to_hba(n) container_of(n, struct ctlr_info, dev)
#define to_drv(n) container_of(n, drive_info_struct, dev)
/* List of controllers which cannot be reset on kexec with reset_devices */
static u32 unresettable_controller[] = {
0x324a103C, /* Smart Array P712m */
0x324b103C, /* SmartArray P711m */
@@ -576,23 +576,45 @@ static u32 unresettable_controller[] = {
0x409D0E11, /* Smart Array 6400 EM */
};
-static int ctlr_is_resettable(struct ctlr_info *h)
+/* List of controllers which cannot even be soft reset */
+static u32 soft_unresettable_controller[] = {
+ 0x409C0E11, /* Smart Array 6400 */
+ 0x409D0E11, /* Smart Array 6400 EM */
+};
+
+static int ctlr_is_hard_resettable(u32 board_id)
{
int i;
for (i = 0; i < ARRAY_SIZE(unresettable_controller); i++)
- if (unresettable_controller[i] == h->board_id)
+ if (unresettable_controller[i] == board_id)
return 0;
return 1;
}
+static int ctlr_is_soft_resettable(u32 board_id)
+{
+ int i;
+
+ for (i = 0; i < ARRAY_SIZE(soft_unresettable_controller); i++)
+ if (soft_unresettable_controller[i] == board_id)
+ return 0;
+ return 1;
+}
+
+static int ctlr_is_resettable(u32 board_id)
+{
+ return ctlr_is_hard_resettable(board_id) ||
+ ctlr_is_soft_resettable(board_id);
+}
+
static ssize_t host_show_resettable(struct device *dev,
struct device_attribute *attr,
char *buf)
{
struct ctlr_info *h = to_hba(dev);
- return snprintf(buf, 20, "%d\n", ctlr_is_resettable(h));
+ return snprintf(buf, 20, "%d\n", ctlr_is_resettable(h->board_id));
}
static DEVICE_ATTR(resettable, S_IRUGO, host_show_resettable, NULL);
@@ -4601,12 +4623,16 @@ static __devinit int cciss_kdump_hard_reset_controller(struct pci_dev *pdev)
* likely not be happy. Just forbid resetting this conjoined mess.
*/
cciss_lookup_board_id(pdev, &board_id);
- if (board_id == 0x409C0E11 || board_id == 0x409D0E11) {
+ if (!ctlr_is_resettable(board_id)) {
dev_warn(&pdev->dev, "Cannot reset Smart Array 640x "
"due to shared cache module.");
return -ENODEV;
}
+ /* if controller is soft- but not hard resettable... */
+ if (!ctlr_is_hard_resettable(board_id))
+ return -ENOTSUPP; /* try soft reset later. */
+
/* Save the PCI command register */
pci_read_config_word(pdev, 4, &command_register);
/* Turn the board off. This is so that later pci_restore_state()
next prev parent reply other threads:[~2011-05-03 20:02 UTC|newest]
Thread overview: 18+ messages / expand[flat|nested] mbox.gz Atom feed top
2011-05-03 19:52 [PATCH 00/16] cciss: May 3, 2011 updates Stephen M. Cameron
2011-05-03 19:52 ` [PATCH 01/16] cciss: add readl after writel in interrupt mask setting code Stephen M. Cameron
2011-05-03 19:53 ` [PATCH 02/16] cciss: do a better job of detecting controller reset failure Stephen M. Cameron
2011-05-03 19:53 ` [PATCH 03/16] cciss: factor out command pool allocation functions Stephen M. Cameron
2011-05-03 19:53 ` [PATCH 04/16] cciss: factor out scatterlist " Stephen M. Cameron
2011-05-03 19:53 ` [PATCH 05/16] cciss: factor out irq request code Stephen M. Cameron
2011-05-03 19:53 ` [PATCH 06/16] cciss: fix reply pool and block fetch table memory leaks Stephen M. Cameron
2011-05-03 19:53 ` [PATCH 07/16] cciss: get rid of message related magic numbers Stephen M. Cameron
2011-05-03 19:53 ` [PATCH 08/16] cciss: increase time to wait for board reset to start Stephen M. Cameron
2011-05-03 19:53 ` [PATCH 09/16] cciss: clarify messages around reset behavior Stephen M. Cameron
2011-05-03 19:53 ` [PATCH 10/16] cciss: increase timeouts for post-reset no-ops Stephen M. Cameron
2011-05-03 19:53 ` [PATCH 11/16] cciss: use new doorbell-bit-5 reset method Stephen M. Cameron
2011-05-03 19:53 ` [PATCH 12/16] cciss: do soft reset if hard reset is broken Stephen M. Cameron
2011-05-03 19:53 ` [PATCH 13/16] cciss: remove superfluous sleeps around reset code Stephen M. Cameron
2011-05-03 19:54 ` Stephen M. Cameron [this message]
2011-05-03 19:54 ` [PATCH 15/16] cciss: do not use bit 2 doorbell reset Stephen M. Cameron
2011-05-03 19:54 ` [PATCH 16/16] cciss: add cciss_tape_cmds module paramter Stephen M. Cameron
2011-05-06 14:29 ` [PATCH 00/16] cciss: May 3, 2011 updates Jens Axboe
Reply instructions:
You may reply publicly to this message via plain-text email
using any one of the following methods:
* Save the following mbox file, import it into your mail client,
and reply-to-all from there: mbox
Avoid top-posting and favor interleaved quoting:
https://en.wikipedia.org/wiki/Posting_style#Interleaved_style
* Reply using the --to, --cc, and --in-reply-to
switches of git-send-email(1):
git send-email \
--in-reply-to=20110503195402.5154.40908.stgit@beardog.cce.hp.com \
--to=scameron@beardog.cce.hp.com \
--cc=akpm@linux-foundation.org \
--cc=axboe@kernel.dk \
--cc=linux-kernel@vger.kernel.org \
--cc=mikem@beardog.cce.hp.com \
--cc=smcameron@yahoo.com \
--cc=thenzl@redhat.com \
/path/to/YOUR_REPLY
https://kernel.org/pub/software/scm/git/docs/git-send-email.html
* If your mail client supports setting the In-Reply-To header
@@ -558,7 +558,7 @@ static void __devinit cciss_procinit(ctlr_info_t *h)
#define to_hba(n) container_of(n, struct ctlr_info, dev)
#define to_drv(n) container_of(n, drive_info_struct, dev)
-/* List of controllers which cannot be reset on kexec with reset_devices */
+/* List of controllers which cannot be hard reset on kexec with reset_devices */
static u32 unresettable_controller[] = {
0x324a103C, /* Smart Array P712m */
via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line
before the message body.
This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox